@@ -285,6 +285,110 @@ void KKDCanvas::mousePressEvent( QMouseEvent* e )
 				}
 				case RightButton:
 				{
+					if( item != NULL )
+					{
+						if( item->selected() )
+						{
+							if( e->state() & ControlButton )
+							{
+								document()->setSelected( item, false, false );
+							}
+						}
+						else
+						{
+							if( e->state() & ControlButton )
+							{
+								document()->setSelected( item, true, false );
+							}
+							else
+							{
+								document()->setSelected( item, true, true );
+							}
+						}
+					}
+					else
+					{
+						if( !( e->state() & ShiftButton ) )
+						{
+							if( !m_paper->selected() )
+							{
+								document()->setSelected( m_paper, true, true );
+							}
+						}
+					}
+					
+					bool locked = true;
+					int type = 0;
+					for( KKDItemBase* i = m_paper->first(); i; i = m_paper->next() )
+					{
+						if( i->selected() && !i->locked() )
+						{
+							if( !i->locked() ) locked = false;
+							type |= ( 1 << i->type() );
+						}
+					}
+					
+					KPopupMenu* menu = new KPopupMenu( this );
+					
+					if( !locked )
+					{
+						action( "edit_cut" )->plug( menu );
+						action( "edit_copy" )->plug( menu );
+						menu->insertSeparator();
+						action( "raise" )->plug( menu );
+						action( "lower" )->plug( menu );
+						menu->insertSeparator();
+						if( type & 1 << KKDItemBase::TextItem 
+						|| type & 1 << KKDItemBase::Zipcode
+						|| type & 1 << KKDItemBase::Address
+						|| type & 1 << KKDItemBase::Addressee
+						)
+						{
+							action( "fontfamily" )->plug( menu );
+							action( "fontsize" )->plug( menu );
+							action( "fontbold" )->plug( menu );
+							action( "fontitalic" )->plug( menu );
+							action( "fontunderline" )->plug( menu );
+							action( "fontstrikeout" )->plug( menu );
+							action( "color" )->plug( menu );
+							menu->insertSeparator();
+						}
+						if( type & 1 << KKDItemBase::TextItem )
+						{
+							action( "aligntop" )->plug( menu );
+							action( "alignmiddle" )->plug( menu );
+							action( "alignbottom" )->plug( menu );
+							action( "alignleft" )->plug( menu );
+							action( "aligncenter" )->plug( menu );
+							action( "alignright" )->plug( menu );
+							menu->insertSeparator();
+						}
+						if( type & 1 << KKDItemBase::Address
+						 || type & 1 << KKDItemBase::Addressee
+						  )
+						{
+							action( "vertical" )->plug( menu );
+							action( "horizontal" )->plug( menu );
+							menu->insertSeparator();
+						}
+
+						if( type & 1 << KKDItemBase::ImageItem )
+						{
+							action( "imageerase" )->plug( menu );
+							menu->insertSeparator();
+						}
+						action( "remove" )->plug( menu );
+						menu->insertSeparator();
+					}
+					if( type == 0 && !m_paper->locked() )
+					{
+						action( "imageerase" )->plug( menu );
+						menu->insertSeparator();
+						action( "edit_paste" )->plug( menu );
+						menu->insertSeparator();
+					}
+					action( "locked" )->plug( menu );
+					menu->exec( e->globalPos() );
 					break;
 				}
 				default:
