@@ -48,6 +48,7 @@ KKDDoc::KKDDoc( KCommandHistory* cmdHistory, QWidget* parent, const char* name )
 	, m_cmdHistory( cmdHistory )
 	, m_cmd( NULL )
 {
+	m_clipboard.setAutoDelete( true );
 }
 
 KKDDoc::~ KKDDoc()
@@ -894,8 +895,20 @@ void KKDDoc::setImage( const QString& p )
 	QImage img( p );
 	setImage( img );
 }
+
+void KKDDoc::eraseImage()
+{
+	QImage img;
+	setImage( img );
+}
 //END   Image
 
+//BEGIN ImagePrint
+bool KKDDoc::imagePrint()
+{
+	return current()->imagePrint();
+}
+
 void KKDDoc::setImagePrint( bool p )
 {
 	terminate();
@@ -917,6 +930,7 @@ void KKDDoc::setImagePrint( bool p )
 	}
 	m_cmd = NULL;
 }
+//END    ImagePrint
 
 //BEGIN Locked
 /**
@@ -957,6 +971,26 @@ void KKDDoc::setLocked( bool l )
 }
 //END   Locked
 
+bool KKDDoc::imagePrintEnabled()
+{
+	QPtrList<KKDItemBase> items( m_selectedItems );
+	for( KKDItemBase* item = items.first(); item; item = items.next() )
+	{
+		if( item->type() == KKDItemBase::Paper ) return true;
+	}
+	return false;
+}
+
+bool KKDDoc::imageEraseEnabled()
+{
+	QPtrList<KKDItemBase> items( m_selectedItems );
+	for( KKDItemBase* item = items.first(); item; item = items.next() )
+	{
+		if( item->type() == KKDItemBase::Paper || item->type() == KKDItemBase::ImageItem ) return true;
+	}
+	return false;
+}
+
 void KKDDoc::insert()
 {
 	terminate();
@@ -993,28 +1027,6 @@ void KKDDoc::remove()
 	m_cmd = NULL;
 }
 
-// bool KKDDoc::print( QPainter& p, const KKContactData *data )
-// {
-// 	KKDItemBase* item = (KKDItemBase*)selectedItems().first();
-// 	if( !item ) return false;
-// 	if( item->parent() != NULL ) item = (KKDItemBase*)item->parent();
-//
-// 	item->drawItem( p, 1, data );
-// 	if( item->firstChild() )
-// 	{
-// 		printEach( item->firstChild(), p, data );
-// 	}
-// 	return true;
-// }
-//
-// void KKDDoc::printEach( QListViewItem* item, QPainter& p, const KKContactData *data )
-// {
-// 	// To draw reverse order( lastChild to firstChild )
-// 	QListViewItem* next = item->nextSibling();
-// 	if( next ) printEach( next, p, data );
-// 	((KKDItemBase*)item)->drawItem( p, 1, data );
-// }
-
 void KKDDoc::cut()
 {
 	copy();
@@ -1049,10 +1061,7 @@ void KKDDoc::copy()
 		}
 		if( i ) m_clipboard.append( i );
 	}
-	if( m_clipboard.count() > 0 )
-	{
-		emit pasteEnabledChanged( true );
-	}
+	emit pasteEnabledChanged();
 }
 
 void KKDDoc::paste()
@@ -1081,12 +1090,13 @@ void KKDDoc::paste()
 				break;
 		}
 		if( !i ) continue;
+		paper->append( i );
 		i->setSelected( true );
+		m_selectedItems.append( i );
 		insertByCommand( paper, i );
 	}
 	insert();
-	emit cutEnabledChanged( true );
-	emit copyEnabledChanged( true );
+	emit selectedChanged();
 }
 
 void KKDDoc::raise()
@@ -1127,3 +1137,16 @@ void KKDDoc::lower()
 
 
 //END Actions
+
+bool KKDDoc::cutable() const
+{
+	bool locked = false;
+	QPtrList<KKDItemBase> items( m_selectedItems );
+	for( KKDItemBase* item = items.first(); item; item = items.next() )
+	{
+		if( item->type() == KKDItemBase::Paper ) return false;
+		if( item->locked() ) locked = true;
+	}
+	return !locked;
+}
+
