@@ -29,6 +29,7 @@
 #include "../kkaddressbook/kkacontactdata.h"
 
 #include <qdom.h>
+#include <qpaintdevicemetrics.h>
 #include <kurl.h>
 #include <kdebug.h>
 #include <kaction.h>
@@ -42,6 +43,8 @@
 #include <kfontdialog.h>
 #include <tkcoloractions.h>
 #include <kprinter.h>
+#include <kmainwindow.h>
+#include <kxmlguiclient.h>
 
 using namespace KKDesigner;
 
@@ -141,6 +144,10 @@ KKDWidget::KKDWidget( KCommandHistory* cmdHistory, QWidget* parent, const char*
 	//END Font related
 
 	new KToggleAction( i18n( "Lock" ), "encrypted", KShortcut(Qt::CTRL + Qt::Key_L), this, SLOT( setLocked() ), actionCollection(), "locked" );
+	
+	new KToggleAction( i18n( "Print Image" ), "imagegallery", KShortcut(Qt::CTRL + Qt::Key_I), this, SLOT( setImagePrint() ), actionCollection(), "imageprint" );
+	
+	new KAction( i18n( "&Erase Image" ), "eraser", KShortcut(), m_doc, SLOT( eraseImage() ), actionCollection(), "imageerase" );
 	//END Edit Menu
 
 // 	new KAction( i18n( "&Subscribe template" ), "sendtemplate", 0, this, SLOT( subscribeTemplate() ), actionCollection(), "subscribetemplate" );
@@ -168,6 +175,8 @@ KKDWidget::KKDWidget( KCommandHistory* cmdHistory, QWidget* parent, const char*
 	connect( m_doc, SIGNAL( selectedChanged() ), this, SLOT( setActionOrientation() ) );
 	connect( m_doc, SIGNAL( selectedChanged() ), this, SLOT( setActionAlign() ) );
 	connect( m_doc, SIGNAL( selectedChanged() ), this, SLOT( setActionColor() ) );
+	connect( m_doc, SIGNAL( selectedChanged() ), this, SLOT( setActionImagePrint() ) );
+	connect( m_doc, SIGNAL( selectedChanged() ), this, SLOT( setActionImageErase() ) );
 
 	connect( m_doc, SIGNAL( lockedChanged() ), this, SLOT( setActionLocked() ) );
 	connect( m_doc, SIGNAL( fontFamilyChanged() ), this, SLOT( setActionFontFamily() ) );
@@ -180,16 +189,14 @@ KKDWidget::KKDWidget( KCommandHistory* cmdHistory, QWidget* parent, const char*
 	connect( m_doc, SIGNAL( alignChanged() ), this, SLOT( setActionAlign() ) );
 	connect( m_doc, SIGNAL( colorChanged() ), this, SLOT( setActionColor() ) );
 
-	setActionLocked();
-	setActionFontFamily();
-	setActionFontSize();
-	setActionFontBold();
-	setActionFontItalic();
-	setActionFontUnderline();
-	setActionFontStrikeOut();
-	setActionOrientation();
-	setActionAlign();
-	setActionColor();
+	connect( m_doc, SIGNAL( imagePrintChanged() ), this, SLOT( setActionImagePrint() ) );
+
+	connect( m_doc, SIGNAL( selectedChanged() ), this, SLOT( setActionCutEnabled() ) );
+	connect( m_doc, SIGNAL( selectedChanged() ), this, SLOT( setActionCopyEnabled() ) );
+	connect( m_doc, SIGNAL( pasteEnabledChanged() ), this, SLOT( setActionPasteEnabled() ) );
+	connect( m_doc, SIGNAL( lockedChanged() ), this, SLOT( setActionCutEnabled() ) );
+	connect( m_doc, SIGNAL( lockedChanged() ), this, SLOT( setActionCopyEnabled() ) );
+	connect( m_doc, SIGNAL( lockedChanged() ), this, SLOT( setActionPasteEnabled() ) );
 }
 
 KKDWidget::~KKDWidget()
@@ -243,6 +250,18 @@ void KKDWidget::fileOpen( const QString& filename, bool blCheckChanged )
 	setEditActionEnabled();
 
 	resetCaption();
+	setActionLocked();
+	setActionFontFamily();
+	setActionFontSize();
+	setActionFontBold();
+	setActionFontItalic();
+	setActionFontUnderline();
+	setActionFontStrikeOut();
+	setActionOrientation();
+	setActionAlign();
+	setActionColor();
+	setActionImagePrint();
+	setActionImageErase();
 }
 
 void KKDWidget::openRecent( const KURL& url )
@@ -315,12 +334,21 @@ void KKDWidget::print()
 			QPtrList< KMainWindow >* list = KMainWindow::memberList;
 			QPtrList<KKAddressBook::KKAContactData> data =  ((KreetingKard*)list->at(0))->printData();
 
-			if( data.count() == 0 ) return;
+			if( data.count() == 0 )
+			{
+				KMessageBox::error( this, i18n( "There is no data that print type is \"Print\"." ) );
+				return;
+			}
 			if( !printer.setup( this ) ) return;
 
 			bool newPage = false;
 			QPainter p;
 			p.begin( &printer );
+
+			QPaintDeviceMetrics metrics( p.device() );
+			QWMatrix matrix( p.worldMatrix() );
+			p.setWorldMatrix( matrix.scale( (double)metrics.width() / (double)paper->size().width(), (double)metrics.height() / (double)paper->size().height() ) );
+
 			for( KKAddressBook::KKAContactData* d = data.first(); d; d = data.next() )
 			{
 				KKAddressBook::KKAAddressData::LocationType l = d->current()->locationType();
@@ -355,8 +383,9 @@ void KKDWidget::print()
 				d->setCurrent( l );
 			}
 			p.end();
+			
 			int ans = KMessageBox::questionYesNo( this, i18n( "Do you change status Print to Printed?" ), i18n( "Print" ) );
-			if( ans == KMessageBox::Ok )
+			if( ans == KMessageBox::Yes )
 			{
 				for( KKAddressBook::KKAContactData* d = data.first(); d; d = data.next() )
 				{
@@ -386,6 +415,10 @@ void KKDWidget::print()
 			bool newPage = false;
 			QPainter p;
 			p.begin( &printer );
+			
+			QPaintDeviceMetrics metrics( p.device() );
+			QWMatrix matrix( p.worldMatrix() );
+			p.setWorldMatrix( matrix.scale( (double)metrics.width() / (double)paper->size().width(), (double)metrics.height() / (double)paper->size().height() ) );
 
 			for( int i = 0; i != printer.numCopies(); i++ )
 			{
@@ -509,6 +542,12 @@ void KKDWidget::setColor()
 	m_doc->setColor( ((TKSelectColorAction*)action( "color" ))->color() );
 	setActionColor();
 }
+
+void KKDWidget::setImagePrint()
+{
+	m_doc->setImagePrint( ((KToggleAction*)action( "imageprint" ))->isChecked() );
+	setActionImagePrint();
+}
 //END Edit Menu
 
 //BEGIN Set action's value
@@ -614,8 +653,38 @@ void KKDWidget::setActionColor()
 {
 	((TKSelectColorAction*)action( "color" ))->setCurrentColor( m_doc->color() );
 }
+
+void KKDWidget::setActionImagePrint()
+{
+	((KToggleAction*)action( "imageprint" ))->setChecked( m_doc->imagePrint() );
+	
+	action( "imageprint" )->setEnabled( !m_filename.isNull() && m_doc->imagePrintEnabled() );
+}
+
+void KKDWidget::setActionImageErase()
+{
+	action( "imageerase" )->setEnabled( !m_filename.isNull() && m_doc->imageEraseEnabled() );
+}
 //END Set action's value
 
+void KKDWidget::setActionCutEnabled()
+{
+	bool enabled = !m_filename.isNull();
+	((KRadioAction*)action( "edit_cut" ))->setEnabled( enabled && m_doc->cutable() );
+}
+
+void KKDWidget::setActionCopyEnabled()
+{
+	bool enabled = !m_filename.isNull();
+	((KRadioAction*)action( "edit_copy" ))->setEnabled( enabled && m_doc->copyable() );
+}
+
+void KKDWidget::setActionPasteEnabled()
+{
+	bool enabled = !m_filename.isNull();
+	((KRadioAction*)action( "edit_paste" ))->setEnabled( enabled && m_doc->pastable() && !m_doc->current()->locked() );
+}
+
 void KKDWidget::subscribeTemplate()
 {
 	if( !checkChangedAndSaved() ) return;
@@ -665,12 +734,12 @@ bool KKDWidget::closable()
 
 void KKDWidget::tabChanged( QWidget* wgt )
 {
-	bool blEnabled = ( wgt == this );
+	bool enabled = ( wgt == this );
 	for( QStringList::ConstIterator it = m_strlActions.begin(); it != m_strlActions.end(); ++it )
 	{
-		setActionEnabled( *it, blEnabled );
+		setActionEnabled( *it, enabled );
 	}
-	if( blEnabled ) setEditActionEnabled();
+	if( enabled ) setEditActionEnabled();
 	resetCaption();
 }
 
@@ -701,41 +770,43 @@ bool KKDWidget::checkChangedAndSaved()
 
 void KKDWidget::setEditActionEnabled()
 {
-	bool blEnabled = !m_filename.isNull();
-	setActionEnabled( "file_save", blEnabled );
-	setActionEnabled( "file_save_as", blEnabled );
-	setActionEnabled( "file_print", blEnabled );
-	setActionEnabled( "file_close", blEnabled );
-	setActionEnabled( "inserttext", blEnabled );
-	setActionEnabled( "insertimage", blEnabled );
-	setActionEnabled( "insertaddressee", blEnabled );
-	setActionEnabled( "insertaddress", blEnabled );
-	setActionEnabled( "insertzipcode", blEnabled );
-	setActionEnabled( "remove", blEnabled );
-	setActionEnabled( "edit_select_all", blEnabled );
+	bool enabled = !m_filename.isNull();
+	setActionEnabled( "file_save", enabled );
+	setActionEnabled( "file_save_as", enabled );
+	setActionEnabled( "file_print", enabled );
+	setActionEnabled( "file_close", enabled );
+	setActionEnabled( "inserttext", enabled );
+	setActionEnabled( "insertimage", enabled );
+	setActionEnabled( "insertaddressee", enabled );
+	setActionEnabled( "insertaddress", enabled );
+	setActionEnabled( "insertzipcode", enabled );
+	setActionEnabled( "remove", enabled );
+	setActionEnabled( "edit_select_all", enabled );
 
-	setActionEnabled( "alignleft", blEnabled );
-	setActionEnabled( "aligncenter", blEnabled );
-	setActionEnabled( "alignright", blEnabled );
-	setActionEnabled( "aligntop", blEnabled );
-	setActionEnabled( "alignmiddle", blEnabled );
-	setActionEnabled( "alignbottom", blEnabled );
+	setActionEnabled( "alignleft", enabled );
+	setActionEnabled( "aligncenter", enabled );
+	setActionEnabled( "alignright", enabled );
+	setActionEnabled( "aligntop", enabled );
+	setActionEnabled( "alignmiddle", enabled );
+	setActionEnabled( "alignbottom", enabled );
 
-	setActionEnabled( "fontfamily", blEnabled );
-	setActionEnabled( "fontsize", blEnabled );
-	setActionEnabled( "fontbold", blEnabled );
-	setActionEnabled( "fontitalic", blEnabled );
-	setActionEnabled( "fontunderline", blEnabled );
-	setActionEnabled( "fontstrikeout", blEnabled );
+	setActionEnabled( "fontfamily", enabled );
+	setActionEnabled( "fontsize", enabled );
+	setActionEnabled( "fontbold", enabled );
+	setActionEnabled( "fontitalic", enabled );
+	setActionEnabled( "fontunderline", enabled );
+	setActionEnabled( "fontstrikeout", enabled );
 
-	setActionEnabled( "color", blEnabled );
-	setActionEnabled( "locked", blEnabled );
+	setActionEnabled( "color", enabled );
+	setActionEnabled( "locked", enabled );
+	setActionEnabled( "imageprint", enabled );
+	setActionEnabled( "imageerase", enabled );
 
-	setActionEnabled( "raise", blEnabled );
-	setActionEnabled( "lower", blEnabled );
-	setActionEnabled( "edit_cut", false );
-	setActionEnabled( "edit_copy", false );
-	setActionEnabled( "edit_paste", false );
-	setActionEnabled( "view_zoom_in", false );
+	setActionEnabled( "raise", enabled );
+	setActionEnabled( "lower", enabled );
+	setActionEnabled( "edit_cut", enabled && ( m_doc->cutable() ) );
+	setActionEnabled( "edit_copy", enabled && ( m_doc->copyable() ) );
+	setActionEnabled( "edit_paste", enabled && m_doc->pastable() );
+	setActionEnabled( "view_zoom_in", enabled );
 	setActionEnabled( "view_zoom_out", false );
 }
